@@ -403,7 +403,8 @@ json_reader_is_value (JsonReader *reader)
   if (reader->priv->current_node == NULL)
     return FALSE;
 
-  return JSON_NODE_HOLDS_VALUE (reader->priv->current_node);
+  return JSON_NODE_HOLDS_VALUE (reader->priv->current_node) ||
+         JSON_NODE_HOLDS_NULL (reader->priv->current_node);
 }
 
 /**
@@ -867,13 +868,6 @@ json_reader_get_int_value (JsonReader *reader)
       return 0;
     }
 
-  if (json_node_get_value_type (node) != G_TYPE_INT64)
-    {
-      json_reader_set_error (reader, JSON_READER_ERROR_INVALID_TYPE,
-                             _("The current position does not hold an integer type"));
-      return 0;
-    }
-
   return json_node_get_int (reader->priv->current_node);
 }
 
@@ -912,13 +906,6 @@ json_reader_get_double_value (JsonReader *reader)
       return 0.0;
     }
 
-  if (json_node_get_value_type (node) != G_TYPE_DOUBLE)
-    {
-      json_reader_set_error (reader, JSON_READER_ERROR_INVALID_TYPE,
-                             _("The current position does not hold a floating point type"));
-      return 0.0;
-    }
-
   return json_node_get_double (reader->priv->current_node);
 }
 
@@ -1002,13 +989,6 @@ json_reader_get_boolean_value (JsonReader *reader)
       return FALSE;
     }
 
-  if (json_node_get_value_type (node) != G_TYPE_BOOLEAN)
-    {
-      json_reader_set_error (reader, JSON_READER_ERROR_INVALID_TYPE,
-                             _("The current position does not hold a boolean type"));
-      return FALSE;
-    }
-
   return json_node_get_boolean (node);
 }
 
